module step
(
	input				clk,			// system clock
	input				rst_n,			// system reset, active low
	input	    [3:0]	sw,	
	input		[2:0]	key,
	inout         		one_wire,
	
	input				fpga_rx,	//UART
	output				fpga_tx,	//UART
	
	output				oled_csn,	//OLCDҺʹ
	output				oled_rst,	//OLCDҺλ
	output				oled_dcn,	//OLCDָ
	output				oled_clk,	//OLCDʱź
	output				oled_dat,	//OLCDź
	
	output				beeper
);



wire	[15:0]	data_out;
//Drive DS18B20Z to get temperature code
DS18B20Z DS18B20Z_uut
(
.clk					(clk			),	// system clock
.rst_n					(rst_n			),	// system reset, active low
.one_wire				(one_wire		),	// ds18b20z one-wire-bus
.data_out				(data_out		)	// ds18b20z data_out
);


// judge sign of temperature
wire temperature_flag = data_out[15:11]? 1'b0:1'b1;
// complement if negative
wire [10:0] temperature_code = temperature_flag? data_out[10:0]:(~data_out[10:0])+1'b1; 
// translate temperature_code to real temperature
wire [20:0] bin_code = temperature_code * 16'd625;
wire [24:0] bcd_code; //ʮλ[23:20],λ[19:16],Сλ[14:12]
//Translate binary code to bcd code
bin_to_bcd bin_to_bcd_uut
(
.rst_n					(rst_n			),	// system reset, active low
.bin_code				(bin_code		),	// binary code
.bcd_code				(bcd_code		)	// bcd code
);


wire [31:0] time_oled;
wire [31:0] temp_oled;
OLED12832 OLED12832_uut
(
.clk					(clk			),	// system clock
.rst_n					(rst_n			),	// system reset, active low
.tim			    	(time_oled		),	// ds18b20z one-wire-bus
.tem    				(temp_oled		),	// ds18b20z data_out
.oled_csn				(oled_csn		),
.oled_rst				(oled_rst		),
.oled_dcn				(oled_dcn		),
.oled_clk				(oled_clk		),
.oled_dat				(oled_dat		)

);



wire rx_data_valid;
wire [7:0]	rx_data;
wire tx_data_valid;
wire [7:0]	tx_data;
//Uart_Bus module
Uart_Bus u1
(	
.clk					(clk			),	//ϵͳʱ 12MHz
.rst_n					(rst_n			),	//ϵͳλЧ
//FPGAUARTоƬ
.uart_rx				(fpga_rx		),	//UART
.rx_data_valid			(rx_data_valid	),	//Ч
.rx_data_out			(rx_data		),	//յ     8λ
//FPGAݸUARTоƬ
.tx_data_valid			(tx_data_valid	),
.tx_data_in				(tx_data		),
.uart_tx				(fpga_tx		)
);


wire [15:0] cycle;
//ݲͬڵcycleֵӦPWMź
PWM #
(
.WIDTH					(16			)	//ensure that 2**WIDTH > cycle
)
PWM_uut
(
.clk					(clk		),
.rst_n					(rst_n		),
.cycle					(cycle		),	//cycle > duty
.duty					(cycle>>1	),	//duty < cycle
.pwm_out				(beeper	)
);


wire [2:0] key_pulse;
key #
(
.N						(3			)	
)
key_s
(
.clk					(clk		),
.rst					(rst_n		),
.key					(key		),	
.key_pulse				(key_pulse	)	
);



control control_s
(
.clk					(clk			),	// system clock
.rst_n					(rst_n			),	// system reset, active low
.sw      				(sw				),	
.key					(key_pulse		),
.temp_in				(bcd_code[23:12]		),	// ds18b20z data_out

.rx_data_valid        	(rx_data_valid	),
.rx_data_in  			(rx_data	 	),

.tx_data_valid        	(tx_data_valid	),
.tx_data_out  			(tx_data	 	),

.time_out				(time_oled		),
.temp_out				(temp_oled		),

.cycle					(cycle			)

);




endmodule